@@ -8,7 +8,6 @@
 #include <QDBusConnection>
 #include <QDBusMessage>
 #include <QDBusPendingReply>
-#include <QDebug>
 #include <polkitqt1-agent-session.h>
 #include <polkitqt1-authority.h>
 
@@ -27,6 +26,7 @@
 #include "interface_putcommand.h"
 #include "interface_renamecommand.h"
 #include "interface_statcommand.h"
+#include "kioadmin_debug.h"
 
 using namespace KIO;
 using namespace std::chrono_literals;
@@ -201,25 +201,34 @@ public:
     bool considerRemembering(ReadAuthorizationRequest request)
     {
         auto previousRequest = s_previousReadAuthorisationRequest.load();
-        while (previousRequest && !previousRequest->isStillRelevant()) {
+        while (!previousRequest || (previousRequest && !previousRequest->isStillRelevant())) {
             // The previousRequest is somewhat useless. We forget about it.
-            if (s_previousReadAuthorisationRequest.compare_exchange_strong(previousRequest, std::nullopt)) {
-                // s_previousReadAuthorisationRequest was successfully set to std::nullopt
-                break;
+            // In normal code we would simply set it to zero/std::nullopt now. Unfortunately we are dealing with multi-threaded code here,
+            // so for synchonization reasons we do not want to have s_previousReadAuthorisationRequest unnecessarily empty even for a split second.
+            // So instead of setting it to std::nullopt unconditionally here, we replace it immediately with the current request if that makes sense.
+            // This fixes a random race condition that could lead to a second password prompt unnecessarily popping up after the first one was cancelled.
+            if (request.isStillRelevant()) {
+                auto requestCopy = request;
+                if (s_previousReadAuthorisationRequest.compare_exchange_strong(previousRequest, requestCopy)) {
+                    // s_previousReadAuthorisationRequest was successfully set to request
+                    return true;
+                }
+            } else {
+                if (s_previousReadAuthorisationRequest.compare_exchange_strong(previousRequest, std::nullopt)) {
+                    // s_previousReadAuthorisationRequest was successfully set to std::nullopt
+                    return false;
+                }
             }
+            // previousRequest was changed from somewhere else. The compare_exchange_strong call above has updated our local version of previousRequest to the
+            // current contents of s_previousReadAuthorisationRequest. So next the while-condition will check again if it is relevant now and try to replace it
+            // if it is not.
         }
 
         if (!request.isStillRelevant()) {
             return false;
         }
 
-        // If s_previousReadAuthorisationRequest is empty, assign the current request to it.
-        std::optional<ReadAuthorizationRequest> nulloptRequest = std::nullopt;
-        if (s_previousReadAuthorisationRequest.compare_exchange_strong(nulloptRequest, request)) {
-            return true;
-        }
-
-        if ((!previousRequest || !previousRequest->result()) && request.result()) {
+        if (!previousRequest->result() && request.result()) {
             // The previousRequest doesn't have a result yet which makes it less useful than a request with a result that isStillRelevant().
             if (s_previousReadAuthorisationRequest.compare_exchange_strong(previousRequest, request)) {
                 return true;
@@ -229,7 +238,7 @@ public:
             return false;
         }
 
-        if (!previousRequest || !previousRequest->isSimilarTo(request)) {
+        if (!previousRequest->isSimilarTo(request)) {
             // The previous and the current request are different. Currently only remembering of a singular request is implemented, so now we do not care about
             // the older request anymore and replace it with the current request.
             if (s_previousReadAuthorisationRequest.compare_exchange_strong(previousRequest, request)) {
@@ -289,7 +298,7 @@ public:
         }
 
         const auto path = reply.arguments().at(0).value<QDBusObjectPath>().path();
-        qDebug() << path;
+        qCDebug(KIOADMIN_LOG) << path;
 
         OrgKdeKioAdminListDirCommandInterface iface(serviceName(), path, QDBusConnection::systemBus(), this);
         connect(&iface, &OrgKdeKioAdminListDirCommandInterface::result, this, &AdminWorker::result);
@@ -316,7 +325,7 @@ public:
 
     WorkerResult open(const QUrl &url, QIODevice::OpenMode mode) override
     {
-        qDebug() << Q_FUNC_INFO;
+        qCDebug(KIOADMIN_LOG) << Q_FUNC_INFO;
         auto request = QDBusMessage::createMethodCall(serviceName(), servicePath(), serviceInterface(), QStringLiteral("file"));
         request << url.toString() << (int)mode;
         auto reply = QDBusConnection::systemBus().call(request);
@@ -367,7 +376,7 @@ public:
 
     WorkerResult read(KIO::filesize_t size) override
     {
-        qDebug() << Q_FUNC_INFO;
+        qCDebug(KIOADMIN_LOG) << Q_FUNC_INFO;
         m_file->read(size);
         execLoop(loop);
         return m_result;
@@ -375,7 +384,7 @@ public:
 
     WorkerResult write(const QByteArray &data) override
     {
-        qDebug() << Q_FUNC_INFO;
+        qCDebug(KIOADMIN_LOG) << Q_FUNC_INFO;
         Q_ASSERT(!m_pendingWrite.has_value());
         m_pendingWrite = data.size();
         m_file->write(data);
@@ -385,7 +394,7 @@ public:
 
     WorkerResult seek(KIO::filesize_t offset) override
     {
-        qDebug() << Q_FUNC_INFO;
+        qCDebug(KIOADMIN_LOG) << Q_FUNC_INFO;
         m_file->seek(offset);
         execLoop(loop);
         return m_result;
@@ -393,7 +402,7 @@ public:
 
     WorkerResult truncate(KIO::filesize_t size) override
     {
-        qDebug() << Q_FUNC_INFO;
+        qCDebug(KIOADMIN_LOG) << Q_FUNC_INFO;
         m_file->truncate(size);
         execLoop(loop);
         return m_result;
@@ -401,7 +410,7 @@ public:
 
     WorkerResult close() override
     {
-        qDebug() << Q_FUNC_INFO;
+        qCDebug(KIOADMIN_LOG) << Q_FUNC_INFO;
         m_file->close();
         execLoop(loop);
         return m_result;
@@ -474,7 +483,7 @@ public:
 
     WorkerResult copy(const QUrl &src, const QUrl &dest, int permissions, JobFlags flags) override
     {
-        qDebug() << Q_FUNC_INFO;
+        qCDebug(KIOADMIN_LOG) << Q_FUNC_INFO;
         auto request = QDBusMessage::createMethodCall(serviceName(), servicePath(), serviceInterface(), QStringLiteral("copy"));
         request << src.toString() << dest.toString() << permissions << static_cast<int>(flags);
         auto reply = QDBusConnection::systemBus().call(request);
@@ -482,7 +491,7 @@ public:
             return toFailure(reply);
         }
         const auto path = reply.arguments().at(0).value<QDBusObjectPath>().path();
-        qDebug() << path;
+        qCDebug(KIOADMIN_LOG) << path;
 
         OrgKdeKioAdminCopyCommandInterface iface(serviceName(), path, QDBusConnection::systemBus(), this);
         connect(&iface, &OrgKdeKioAdminCopyCommandInterface::result, this, &AdminWorker::result);
@@ -494,7 +503,7 @@ public:
 
     WorkerResult get(const QUrl &url) override
     {
-        qDebug() << Q_FUNC_INFO;
+        qCDebug(KIOADMIN_LOG) << Q_FUNC_INFO;
         auto request = QDBusMessage::createMethodCall(serviceName(), servicePath(), serviceInterface(), QStringLiteral("get"));
         request << url.toString();
         auto reply = QDBusConnection::systemBus().call(request);
@@ -502,7 +511,7 @@ public:
             return toFailure(reply);
         }
         const auto path = reply.arguments().at(0).value<QDBusObjectPath>().path();
-        qDebug() << path;
+        qCDebug(KIOADMIN_LOG) << path;
 
         OrgKdeKioAdminGetCommandInterface iface(serviceName(), path, QDBusConnection::systemBus(), this);
         connect(&iface, &OrgKdeKioAdminGetCommandInterface::data, this, [this](const QByteArray &blob) {
@@ -522,7 +531,7 @@ public:
     {
         Q_UNUSED(isFile);
 
-        qDebug() << Q_FUNC_INFO;
+        qCDebug(KIOADMIN_LOG) << Q_FUNC_INFO;
         auto request = QDBusMessage::createMethodCall(serviceName(), servicePath(), serviceInterface(), QStringLiteral("del"));
         request << url.toString();
         auto reply = QDBusConnection::systemBus().call(request);
@@ -541,7 +550,7 @@ public:
 
     WorkerResult mkdir(const QUrl &url, int permissions) override
     {
-        qDebug() << Q_FUNC_INFO;
+        qCDebug(KIOADMIN_LOG) << Q_FUNC_INFO;
         auto request = QDBusMessage::createMethodCall(serviceName(), servicePath(), serviceInterface(), QStringLiteral("mkdir"));
         request << url.toString() << permissions;
         auto reply = QDBusConnection::systemBus().call(request);
@@ -560,7 +569,7 @@ public:
 
     WorkerResult rename(const QUrl &src, const QUrl &dest, JobFlags flags) override
     {
-        qDebug() << Q_FUNC_INFO;
+        qCDebug(KIOADMIN_LOG) << Q_FUNC_INFO;
         auto request = QDBusMessage::createMethodCall(serviceName(), servicePath(), serviceInterface(), QStringLiteral("rename"));
         request << src.toString() << dest.toString() << static_cast<int>(flags);
         auto reply = QDBusConnection::systemBus().call(request);
@@ -581,7 +590,7 @@ public:
 
     WorkerResult chmod(const QUrl &url, int permissions) override
     {
-        qDebug() << Q_FUNC_INFO;
+        qCDebug(KIOADMIN_LOG) << Q_FUNC_INFO;
         auto request = QDBusMessage::createMethodCall(serviceName(), servicePath(), serviceInterface(), QStringLiteral("chmod"));
         request << url.toString() << permissions;
         auto reply = QDBusConnection::systemBus().call(request);
@@ -600,7 +609,7 @@ public:
 
     WorkerResult chown(const QUrl &url, const QString &owner, const QString &group) override
     {
-        qDebug() << Q_FUNC_INFO;
+        qCDebug(KIOADMIN_LOG) << Q_FUNC_INFO;
         auto request = QDBusMessage::createMethodCall(serviceName(), servicePath(), serviceInterface(), QStringLiteral("chown"));
         request << url.toString() << owner << group;
         auto reply = QDBusConnection::systemBus().call(request);
@@ -619,7 +628,7 @@ public:
 
     // WorkerResult setModificationTime(const QUrl &url, const QDateTime &mtime) override
     // {
-    //     qDebug() << Q_FUNC_INFO;
+    //     qCDebug(KIOADMIN_LOG) << Q_FUNC_INFO;
     //     return WorkerResult::pass();
     // }
 
@@ -650,19 +659,19 @@ public:
 private Q_SLOTS:
     void entry(const KIO::UDSEntry &entry)
     {
-        qDebug() << Q_FUNC_INFO << entry;
+        qCDebug(KIOADMIN_LOG) << Q_FUNC_INFO << entry;
         statEntry(entry);
     }
 
     void entries(const KIO::UDSEntryList &list)
     {
-        qDebug() << Q_FUNC_INFO;
+        qCDebug(KIOADMIN_LOG) << Q_FUNC_INFO;
         listEntries(list);
     }
 
     void result(int error, const QString &errorString)
     {
-        qDebug() << "RESULT" << error << errorString;
+        qCDebug(KIOADMIN_LOG) << "RESULT" << error << errorString;
         if (error != KJob::NoError) {
             m_result = WorkerResult::fail(error, errorString);
         } else {
